@@ -1,6 +1,6 @@
 package Net::SSH2;
 
-our $VERSION = '0.59_02';
+our $VERSION = '0.59_03';
 
 use 5.006;
 use strict;
@@ -241,6 +241,16 @@ sub new {
     return $self;
 }
 
+sub die_with_error {
+    my $self = shift;
+    if (my ($code, $name, $string) = $self->error) {
+        die join(": ", @_, "$string ($code $name)");
+    }
+    else {
+        die join(": ", @_, "no libssh2 error registered");
+    }
+}
+
 sub method {
     my $self = shift;
     my $method_type = shift;
@@ -265,14 +275,14 @@ sub connect {
         } elsif(ref $sock) {
             # handled below
         } else {
-            @_ = ($sock, getservbyname(qw(ssh tcp)) || 22);
+            @_ = ($sock, 'ssh');
         }
     }
 
     my %opts = splice @_, 2;
     if (%opts) {
         $connect_opts_warned++ or
-            carp "passing options to connect is deprectated";
+            carp "passing options to connect is deprecated";
         $self->timeout($opts{Timeout}) if $opts{Timeout};
         if ($opts{Compress} and
             ($self->version)[1] >= 0x10500) {
@@ -280,18 +290,18 @@ sub connect {
         }
     }
 
+    my ($remote_hostname, $remote_port);
     if (@_ == 2) {
-        my $timeout = $self->timeout;
-        my $blocking = $self->blocking;
-        $sock = $socket_class->new( PeerHost => $_[0],
-                                    PeerPort => $_[1],
-                                    Blocking => $blocking,
-                                    Timeout => $timeout );
+        $remote_hostname = $_[0];
+        $remote_port = getservbyname($_[1] || 'ssh', 'tcp') || 22;
+        $sock = $socket_class->new( PeerHost => $remote_hostname,
+                                    PeerPort => $remote_port,
+                                    Blocking => $self->blocking,
+                                    Timeout => $self->timeout );
         unless ($sock) {
             $self->_set_error(LIBSSH2_ERROR_SOCKET_NONE(), "Unable to connect to remote host: $!");
             goto error;
         }
-
         $sock->sockopt(SO_LINGER, pack('SS', 0, 0));
     }
 
@@ -309,8 +319,16 @@ sub connect {
         $fd = Win32API::File::FdGetOsFHandle($fd);
     }
 
+    {
+        local ($@, $SIG{__DIE__});
+        $remote_port = eval { $sock->peerport }
+            unless defined $remote_port;
+        $remote_hostname = eval { $sock->peername } || 22
+            unless defined $remote_hostname;
+    }
+
     # pass it in, do protocol
-    return $self->_startup($fd, $sock);
+    return $self->_startup($fd, $sock, $remote_hostname, $remote_port);
 
  error:
     unless (defined wantarray) {
@@ -451,6 +469,39 @@ sub auth_password_interact {
     return $rc;
 }
 
+sub check_remote_hostkey {
+    my ($self, $path, $policy) = @_;
+    my $remote_hostname = $self->remote_hostname;
+    croak("remote_hostname unknown: in order to use check_remote_hostkey the peer host name ".
+          "must be given (or discoverable) at connect time")
+        unless defined $remote_hostname;
+
+    unless (defined $path) {
+        my $home = $ENV{HOME} || (getpwuid($<))[7];
+        unless (defined $home) {
+            $self->_set_error(LIBSSH2_ERROR_FILE(), "Unable to determine known_hosts location");
+            return;
+        }
+        require File::Spec;
+        $path = File::Spec->catfile($home, '.ssh', 'known_hosts');
+    }
+
+    my $kh = $self->known_hosts or return;
+    $kh->readfile($path) or return;
+
+    my ($key, $type) = $self->remote_hostkey;
+    my $flags = ( LIBSSH2_KNOWNHOST_TYPE_PLAIN() |
+                  LIBSSH2_KNOWNHOST_KEYENC_RAW() |
+                  (($type + 1) << LIBSSH2_KNOWNHOST_KEY_SHIFT()) );
+
+    my $check = $kh->check($remote_hostname, $self->remote_port, $key, $flags);
+    $check == LIBSSH2_KNOWNHOST_CHECK_MATCH() and return 1;
+
+    # TODO: implement other policies!
+    $self->_set_error(LIBSSH2_ERROR_KNOWN_HOSTS(), 'Unable to verify remote host key');
+    ()
+}
+
 sub scp_get {
     my ($self, $remote, $path) = @_;
     $path = basename $remote if not defined $path;
@@ -740,7 +791,7 @@ Establish the SSH connection calling the method C<connect>.
 =item 4
 
 Check the remote host public key, typically comparing it to the one
-stored in /etc/ssh/known_hosts. See L<Net::SSH2::KnownHosts>.
+stored in C</etc/ssh/known_hosts>. See L<Net::SSH2::KnownHosts>.
 
 =item 5
 
@@ -840,6 +891,18 @@ returns (code, error name, error string).
 Note that the returned error value is only meaningful after some other
 method indicates an error by returning false.
 
+=head2 die_with_error ( [message] )
+
+Calls C<die> with the given message with the error information from
+the object attached.
+
+For instance:
+
+  $ssh2->connect("ajhkfhdklfjhklsjhd", 22)
+    or $ssh2->die_with_error;
+  # dies as:
+  #  Unable to connect to remote host: Invalid argument (-1 LIBSSH2_ERROR_SOCKET_NONE)
+
 =head2 sock
 
 Returns a reference to the underlying L<IO::Socket> object (usually a
@@ -1073,6 +1136,10 @@ Returns the public key of the remote host and its type which is one of
 C<LIBSSH2_HOSTKEY_TYPE_RSA>, C<LIBSSH2_HOSTKEY_TYPE_DSS>, or
 C<LIBSSH2_HOSTKEY_TYPE_UNKNOWN>.
 
+=head2 check_remote_hostkey( known_hosts_path, policy )
+
+Looks for the remote host key in the given file.
+
 =head2 auth_list ( [username] )
 
 Get a list (or comma-separated string in scalar context) of authentication
